/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 * Copyright (C) 2008 INdT - Instituto Nokia de Tecnologia
 * Copyright (C) 2009-2010 ProFUSION embedded systems
 * Copyright (C) 2009-2010 Samsung Electronics
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "RenderThemeWiiU.h"
#include "Scrollbar.h"
#include "ScrollbarThemeWiiU.h"
#include "ScrollbarThemeAssetsWiiU.h"
#include "ByteArray.h"

#include "NotImplemented.h"
#include <stdio.h>
#include <wtf/StdLibExtras.h>
#define ARROW_WIDTH 16
#define ARROW_HEIGHT 16

//#define LR_ARROW_WIDTH 8
//#define LR_ARROW_HEIGHT 8

/*static const unsigned char up_arrow[] = {
		0x18,
		0x3C,
		0x7E,
		0xFF };

static const unsigned char down_arrow[] = {
		0xFF,
		0x7E,
		0x3C,
		0x18 };

static const unsigned char right_arrow[] = {
		0x20,
		0x30,
		0x38,
		0x3C,
		0x3C,
		0x38,
		0x30,
		0x20
};

static const unsigned char left_arrow[] = {
		0x04,
		0x0C,
		0x1C,
		0x3C,
		0x3C,
		0x1C,
		0x0C,
		0x04
};*/

namespace WebCore {

#if !PLATFORM(WIIU)
char ScrollbarThemeWiiU_assetsPath[1024] = {0};

void ScrollbarThemeWiiU::setAssetsPath(const char *set)
{
	if (set)
		strncpy(ScrollbarThemeWiiU_assetsPath, set, sizeof(ScrollbarThemeWiiU_assetsPath));
	else
		ScrollbarThemeWiiU_assetsPath[0] = 0;
}

std::string ScrollbarThemeWiiU::prependAssetsPath(const char *path)
{
	std::string fullpath = ScrollbarThemeWiiU_assetsPath;
	if (path) fullpath.append(path);
	return fullpath;
}
#endif

//NOTE -- We're going to let this leak on app-exit now, since leaving it as a
//			static var causes it to not be created on the same thread as it
//			gets destroyed on.
ScrollbarThemeWiiU * g_ScrollBarTheme = 0;
ScrollbarTheme* ScrollbarTheme::nativeTheme()
{
    if (!g_ScrollBarTheme)
	{
        g_ScrollBarTheme = new ScrollbarThemeWiiU;
		ADD_TO_STATIC_LOCAL(g_ScrollBarTheme);
	}

    return g_ScrollBarTheme;
}

ScrollbarThemeWiiU::ScrollbarThemeWiiU()
{
    m_up = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarVUpBmp, sizeof(scrollBarVUpBmp));
    m_down = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarVDownBmp, sizeof(scrollBarVDownBmp));
    m_left = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarHUpBmp, sizeof(scrollBarHUpBmp));
    m_right = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarHDownBmp, sizeof(scrollBarHDownBmp));

    m_verticalBackground = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarVBackgroundBmp, sizeof(scrollBarVBackgroundBmp));
    m_horizontalBackground = RenderThemeWiiU::loadRenderThemeImageFromMemory(scrollBarHBackgroundBmp, sizeof(scrollBarHBackgroundBmp));
}

ScrollbarThemeWiiU::~ScrollbarThemeWiiU()
{
	m_up = 0;
	m_down = 0;
	m_left = 0;
	m_right = 0;
	m_verticalBackground = 0;
	m_horizontalBackground = 0;
}

int ScrollbarThemeWiiU::scrollbarThickness(ScrollbarControlSize controlSize)
{
    return 16;
}

bool ScrollbarThemeWiiU::hasThumb(Scrollbar* scrollbar)
{
    return thumbLength(scrollbar) > 0;
}

IntRect ScrollbarThemeWiiU::backButtonRect(Scrollbar* scrollbar, ScrollbarPart part, bool)
{
    if (part == BackButtonEndPart)
        return IntRect();

    // Our desired rect is essentially 17x17.

    // Our actual rect will shrink to half the available space when
    // we have < 34 pixels left.  This allows the scrollbar
    // to scale down and function even at tiny sizes.
    int thickness = scrollbarThickness();
    if (scrollbar->orientation() == HorizontalScrollbar)
        return IntRect(scrollbar->x(), scrollbar->y(),
                       scrollbar->width() < 2 * thickness ? scrollbar->width() / 2 : thickness, thickness);
    return IntRect(scrollbar->x(), scrollbar->y(),
                   thickness, scrollbar->height() < 2 * thickness ? scrollbar->height() / 2 : thickness);
}

IntRect ScrollbarThemeWiiU::forwardButtonRect(Scrollbar* scrollbar, ScrollbarPart part, bool)
{
    if (part == ForwardButtonStartPart)
        return IntRect();

    // Our desired rect is essentially 17x17.

    // Our actual rect will shrink to half the available space when
    // we have < 34 pixels left.  This allows the scrollbar
    // to scale down and function even at tiny sizes.
    int thickness = scrollbarThickness();
    if (scrollbar->orientation() == HorizontalScrollbar) {
        int w = scrollbar->width() < 2 * thickness ? scrollbar->width() / 2 : thickness;
        return IntRect(scrollbar->x() + scrollbar->width() - w, scrollbar->y(), w, thickness);
    }

    int h = scrollbar->height() < 2 * thickness ? scrollbar->height() / 2 : thickness;
    return IntRect(scrollbar->x(), scrollbar->y() + scrollbar->height() - h, thickness, h);
}

IntRect ScrollbarThemeWiiU::trackRect(Scrollbar* scrollbar, bool)
{
    int thickness = scrollbarThickness();
    if (scrollbar->orientation() == HorizontalScrollbar) {
        if (scrollbar->width() < 2 * thickness)
            return IntRect();
        return IntRect(scrollbar->x() + thickness, scrollbar->y(), scrollbar->width() - 2 * thickness, thickness);
    }
    if (scrollbar->height() < 2 * thickness)
        return IntRect();
    return IntRect(scrollbar->x(), scrollbar->y() + thickness, thickness, scrollbar->height() - 2 * thickness);
}

void ScrollbarThemeWiiU::paintButton(GraphicsContext* context, Scrollbar* scrollbar, const IntRect& rect, ScrollbarPart part)
{
    context->save();
    bool start = (part == BackButtonStartPart);
    int x1 = rect.x();
    int x2 = rect.x() + rect.width();
    int y1 = rect.y();
    int y2 = rect.y() + rect.height();
    if (start) {
    	if (scrollbar->orientation() == HorizontalScrollbar) {
    		int x = rect.x() + (( rect.width () - 1 - ARROW_WIDTH) / 2);
    		int y = rect.y() + (( rect.height() - 1 - ARROW_HEIGHT) / 2);
            context->drawImage(m_left.get(), ColorSpaceDeviceRGB, FloatRect(x, y, ARROW_WIDTH, ARROW_HEIGHT)); 
    	} else {
    		int x = rect.x() + (( rect.width () - 1 - ARROW_WIDTH) / 2);
    		int y = rect.y() + (( rect.height() - 1 - ARROW_HEIGHT) / 2);
            context->drawImage(m_up.get(), ColorSpaceDeviceRGB, FloatRect(x, y, ARROW_WIDTH, ARROW_HEIGHT)); 
    	}
    } else {
    	if (scrollbar->orientation() == HorizontalScrollbar) {
    		int x = rect.x() + (( rect.width () - 1 - ARROW_WIDTH) / 2);
    		int y = rect.y() + (( rect.height() - 1 - ARROW_HEIGHT) / 2);
            context->drawImage(m_right.get(), ColorSpaceDeviceRGB, FloatRect(x, y, ARROW_WIDTH, ARROW_HEIGHT)); 
    	} else {
    		int x = rect.x() + (( rect.width () - 1 - ARROW_WIDTH) / 2);
    		int y = rect.y() + (( rect.height() - 1 - ARROW_HEIGHT) / 2);
            context->drawImage(m_down.get(), ColorSpaceDeviceRGB, FloatRect(x, y, ARROW_WIDTH, ARROW_HEIGHT)); 
    	}
    }
    context->restore();
}

void ScrollbarThemeWiiU::paintThumb(GraphicsContext* context, Scrollbar* scrollbar, const IntRect& rect)
{
    context->save();
    int x1 = rect.x();
    int x2 = rect.x() + rect.width();
    int y1 = rect.y();
    int y2 = rect.y() + rect.height();

    if (scrollbar->orientation() == HorizontalScrollbar) {
    	context->setFillColor(Color::darkGray, ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect(rect));
    	x1++;
    	x2--;
    	context->setFillColor(Color(255,255,255.255), ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect (x1, y1, x2-x1, y2-y1));
    	y1++;
    	x1++;
    	context->setFillColor(Color(20,20,20.255), ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect (x1, y1, x2-x1, y2-y1));
    	x2--;
    	y2--;
    	context->setFillColor(Color(217,217,217, 255), ColorSpaceDeviceRGB);
    	context->fillRect(FloatRect(x1, y1, x2-x1, y2-y1));
    } else {
    	context->setFillColor(Color::darkGray, ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect(rect));
    	x1++;
    	x2--;
    	context->setFillColor(Color(255,255,255.255), ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect (x1, y1, x2-x1, y2-y1));
    	y1++;
    	x1++;
    	context->setFillColor(Color(20,20,20.255), ColorSpaceDeviceRGB);
    	context->fillRect (FloatRect (x1, y1, x2-x1, y2-y1));
    	x2--;
    	y2--;
    	context->setFillColor(Color(217,217,217, 255), ColorSpaceDeviceRGB);
    	context->fillRect(FloatRect(x1, y1, x2-x1, y2-y1));
    }
    context->restore();
}


void ScrollbarThemeWiiU::paintTrackBackground(GraphicsContext* context, Scrollbar* scrollbar, const IntRect& rect)
{
    // Just assume a forward track part.  We only paint the track as a single piece when there is no thumb.
    if (!hasThumb(scrollbar))
        paintTrackPiece(context, scrollbar, rect, ForwardTrackPart);
}

bool ScrollbarThemeWiiU::invalidateOnMouseEnterExit()
{
    return false;
}

void ScrollbarThemeWiiU::themeChanged()
{
}

bool ScrollbarThemeWiiU::shouldCenterOnThumb(Scrollbar*, const PlatformMouseEvent& evt)
{
    return false; //evt.shiftKey() && evt.button() == LeftButton;
}

void ScrollbarThemeWiiU::paintTrackPiece(GraphicsContext* context, Scrollbar* scrollbar, const IntRect& rect, ScrollbarPart partType)
{
    if (scrollbar->orientation() == HorizontalScrollbar) {
        context->drawImage(m_horizontalBackground.get(), ColorSpaceDeviceRGB, FloatRect(rect)); 
    } else {
        context->drawImage(m_verticalBackground.get(), ColorSpaceDeviceRGB, FloatRect(rect)); 
    }
}


}
