
#ifndef PLATFORMOS_H
#define PLATFORMOS_H

#include <cstring> // for size_t
#include <stdarg.h> // for va_list
#include <vector>
#include <string>

namespace LibPlatform {
	namespace OS {
		typedef void *DynLoad_ModuleHandle;

#ifdef _WIIU_

		typedef signed long long Time;
		typedef unsigned Tick;

		/**
		Handle for sdk alarm structures
		*/
		class Alarm {
		public:
			Alarm(void);
			~Alarm(void);
			void *data;
		};

		typedef struct AtomicVar {
			union {
				unsigned u32;
				signed s32;
				void *ptr32;
			} u;
		} AtomicVar;

		struct Message {
			void *message;
			unsigned data0;
			unsigned data1;
			unsigned data2;
		};

		/**
		Handle for sdk message queues
		*/
		class MessageQueue {
		public:
			MessageQueue(void);
			~MessageQueue(void);
			void *data;
		};

#endif // _WIIU_

		void SetReportCallback(int (*cb)(const char*, va_list));
		void Report(const char *format, ...);
		void VReport(const char *format, va_list args);
		unsigned GetConsoleType(void);
		signed DynLoad_Acquire(char const *module_name, void *module_handle);
		signed DynLoad_FindExport(void *module_handle, signed type_is_data, char const *export_name, void **return_address);
		void DynLoad_Release(void *module_handle);

#ifdef _WIIU_

		Tick MillisecondsToTicks(double milliseconds);
		double GetSystemTimeMillisecondsPrecise();
		void SleepMilliseconds(double milliseconds);
		void SleepMicroseconds(double milliseconds);
		void CreateAlarm(Alarm *alarm);
		void CancelAlarm(Alarm *alarm);
		void SetAlarm(Alarm *alarm, Time ticks_delay, void (*)(void *, void *));
		signed IncAtomic(volatile AtomicVar *pointer);
		signed DecAtomic(volatile AtomicVar *pointer);
		unsigned GetCoreId(void);
		void GetCodegenVirtAddrRange(unsigned *codegenva, size_t *codegensize);
		void SwitchSecCodeGenMode(bool bExecute);
		void GetStack(void **base, void **end);
		void InitMessageQueue(MessageQueue *message_queue, Message *message, signed count);
		signed SendMessage(MessageQueue *message_queue, Message *message, signed flags);
		signed ReceiveMessage(MessageQueue *message_queue, Message *message, signed flags);

		void StoreLaunchArguments(int argc, char* argv[]);
		std::vector<std::string>& GetLaunchArguments();

#endif // _WIIU_
	}
}

#endif // PLATFORMOS_H
