/*---------------------------------------------------------------------------*

  Copyright 2010-2011 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
// -----------------------------------------------------------------------------
//  wksupGamePad.h
//
// -----------------------------------------------------------------------------

#ifndef __WKSUP_GAMEPAD_H__
#define __WKSUP_GAMEPAD_H__

#ifdef __cplusplus
extern "C" {
#endif

/// @addtogroup wksupGamePad 
/// @{

#define wksupMOVColorBuffer                   wksupGamePadColorBuffer
#define wksupMOVContextState                  wksupGamePadContextState
#define wksupMOVInit                          wksupGamePadInit
#define wksupMOVShutdown                      wksupGamePadShutdown
#define wksupMOVBeforeRender                  wksupGamePadBeforeRender
#define wksupMOVDoneRender                    wksupGamePadDoneRender
#define wksupMOVSetDefaultRenderTarget        wksupGamePadSetDefaultRenderTarget

/// \brief Set up GamePad render/depth/scan buffers.
///
/// It must be called before calling any other GamePad functions. \n
/// argv can take follow argment.
///
///  -WKSUP_GAMEPAD_WIDTH n       (set width to n) \n
///  -WKSUP_GAMEPAD_HEIGHT n      (set height to n) \n
///  -WKSUP_GAMEPAD_CB_FORMAT str (set color buffer format to 8_8_8_8 or 10_10_10_2) \n
///  -WKSUP_GAMEPAD_DB_FORMAT str (set depth buffer format to 16, 32F, 8_24, X24_8_32F) \n
///  -WKSUP_GAMEPAD_AA_MODE n     (set AA mode to 0/1/2/3) \n
///
/// Set up defaults for all options (854/480/8_8_8_8/32F/no AA). \n
/// Unknown args should be ignored.  Leave args unchanged.
///
/// \param argc number of argments
/// \param argv argment lists
//
void wksupGamePadInit(int argc, char *argv[], bool depthOn=false);

/// \brief Release allocated buffers.
///
void wksupGamePadShutdown(void);

/// \brief Release allocated buffers.
///
void wksupGamePadReleaseForeground(void);

/// \brief Reacquire allocated buffers.
///
void wksupGamePadAcquiredForeground(void);

/// Set wksup GamePad context to current.
/// Need to call this API before Drawing with wksup GamePad context created in wksupGamePadInit.
void wksupGamePadSetContextState(void);

/// \brief Change state for GamePad
/// Functionality may change in the future.
///
void wksupGamePadBeforeRender(void);

/// \brief Copy render buffer to scan buffers.
/// Note that it does not swap and wait for the new frame to appear.
///
void wksupGamePadDoneRender(void);

/// @}  // wksupGamePad

#ifdef __cplusplus
}
#endif

#endif /// __WKSUP_GAMEPAD_H__
